@@ -1,17 +1,16 @@
 package Workflow::State;
 
-# $Id$
-
 use warnings;
 use strict;
 use base qw( Workflow::Base );
 use Log::Log4perl qw( get_logger );
 use Workflow::Condition::Evaluate;
 use Workflow::Exception qw( workflow_error );
+use Exception::Class;
 use Workflow::Factory qw( FACTORY );
 use English qw( -no_match_vars );
 
-$Workflow::State::VERSION = '1.42';
+$Workflow::State::VERSION = '1.43';
 
 my @FIELDS   = qw( state description type );
 my @INTERNAL = qw( _test_condition_count _factory );
@@ -69,7 +68,17 @@ sub get_available_action_names {
 sub is_action_available {
     my ( $self, $wf, $action_name ) = @_;
     eval { $self->evaluate_action( $wf, $action_name ) };
-    return ( !$EVAL_ERROR );
+
+    # Everything is fine
+    return 1 unless( $EVAL_ERROR );
+
+    # We got an exception, check if it is a Workflow::Exception
+    return 0 if (Exception::Class->caught('Workflow::Exception'));
+
+    $EVAL_ERROR->rethrow() if (ref $EVAL_ERROR);
+
+    croak $EVAL_ERROR;
+
 }
 
 sub clear_condition_cache {
@@ -172,26 +181,56 @@ sub evaluate_action {
             eval { $condition->evaluate($wf) };
             if ($EVAL_ERROR) {
 
-                # TODO: We may just want to pass the error up
-                # without wrapping it...
-                $self->{'_condition_result_cache'}->{$orig_condition} = 0;
-                if ( !$opposite ) {
-                    workflow_error "No access to action '$action_name' in ",
-                        "state '$state' because: $EVAL_ERROR";
+                # Check if this is a Workflow::Exception::Condition
+                if (Exception::Class->caught('Workflow::Exception::Condition')) {
+                    # TODO: We may just want to pass the error up
+                    # without wrapping it...
+                    $self->{'_condition_result_cache'}->{$orig_condition} = 0;
+                    if ( !$opposite ) {
+                        $log->is_debug
+                            && $log->debug("No access to action '$action_name', condition " .
+                             "'$orig_condition' failed because ' . $EVAL_ERROR");
+
+                        workflow_error "No access to action '$action_name' in ",
+                            "state '$state' because: $EVAL_ERROR";
+                    } else {
+                        $log->is_debug
+                            && $log->debug("opposite condition '$orig_condition' failed because ' . $EVAL_ERROR");
+                    }
                 } else {
                     $log->is_debug
-                        && $log->debug("Opposite and condition failed");
+                        && $log->debug("Got uncatchable exception in condition $condition_name ");
+
+                    # if EVAL_ERROR is an execption object rethrow it
+                    $EVAL_ERROR->rethrow() if (ref $EVAL_ERROR ne'');
+
+                    # if it is a string (bubbled up from die/croak), make an Exception Object
+                    # For briefness, we just send back the first line of EVAL
+                    my @t = split /\n/, $EVAL_ERROR;
+                    my $ee = shift @t;
+                    Exception::Class::Base->throw( error
+                        => "Got unknown exception while handling condition '$condition_name' / " . $ee );
                 }
             } else {
                 $self->{'_condition_result_cache'}->{$orig_condition} = 1;
                 if ($opposite) {
+
+                    $log->is_debug
+                        && $log->debug(
+                            "No access to action '$action_name', condition '$orig_condition' ".
+                            "did NOT failed but opposite requested");
+
                     workflow_error "No access to action '$action_name' in ",
                         "state '$state' because condition ",
                         "$orig_condition did NOT fail and we ",
                         "are checking $condition_name.";
                 } else {
-                    $log->is_debug
-                        && $log->debug(" Opposite false and condition OK");
+
+                    $log->is_debug &&
+                        $log->debug(
+                            "condition '$orig_condition' failed, because '$EVAL_ERROR', " .
+                            "but opposite requested");
+
                 }
             }
         }
@@ -517,7 +556,7 @@ state, but if it is not, you are fine to come back and try again later
 if the action is available. This behaviour can be achived by setting the
 'may_stop' property to yes, which will cause Workflow to just quietly
 stop automatic execution if it does not have a single action to execute.
- 
+
 =head1 PUBLIC METHODS
 
 =head3 get_conditions( $action_name )
@@ -552,7 +591,7 @@ Returns list of all action names available in this state.
 Returns all actions names that are available given the data in
 C<$workflow>. Each action name returned will return true from
 B<is_action_available()>.
-$group is optional parameter. If it is set, additional check for group 
+$group is optional parameter. If it is set, additional check for group
 membership will be performed.
 
 =head3 get_next_state( $action_name, [ $action_return ] )
